/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3) 
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details. 
 */

package org.dive4elements.river.exports.fixings;

import au.com.bytecode.opencsv.CSVWriter;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;

import org.dive4elements.river.artifacts.FLYSArtifact;

import org.dive4elements.river.artifacts.access.FixAccess;

import org.dive4elements.river.artifacts.math.fitting.Function;
import org.dive4elements.river.artifacts.math.fitting.FunctionFactory;

import org.dive4elements.river.artifacts.model.CalculationResult;
import org.dive4elements.river.artifacts.model.Parameters;

import org.dive4elements.river.artifacts.model.fixings.FixResult;

import org.dive4elements.river.exports.AbstractExporter;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;

import org.apache.log4j.Logger;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

/** Export result of fixation analysis. */
public class FixATExport extends AbstractExporter {

    /** Private logger. */
    private static Logger logger =
        Logger.getLogger(FixATExport.class);

    protected Function function;
    protected Parameters parameters;


    @Override
    public void doOut(ArtifactAndFacet bundle, Document attr, boolean visible) {
        logger.debug("AT Export doOut().");
        Object data = bundle.getData(context);
        if (data instanceof CalculationResult) {
            CalculationResult cr = (CalculationResult)data;
            Object resData = cr.getData();
            if (resData instanceof FixResult) {
                this.parameters = ((FixResult)resData).getParameters();
            }
        }
        else {
            logger.debug("No CalculationResult found for AT export.");
            return;
        }
        FixAccess access = new FixAccess((FLYSArtifact)this.master, context);
        String f = access.getFunction();
        if (f == null || f.length() == 0) {
            logger.debug("No function found for AT export.");
            return;
        }
        this.function = FunctionFactory.getInstance().getFunction(f);
    }

    @Override
    public void generate() throws IOException {
        if (this.function == null || this.parameters == null) {
            logger.debug("No function or paramters for AT export.");
            return;
        }

        Writer writer = new OutputStreamWriter(out, DEFAULT_CSV_CHARSET);

        FixAccess access = new FixAccess((FLYSArtifact)this.master, context);
        FixATWriter atWriter = new FixATWriter(this.function, this.parameters);
        NodeList nodes = request.getElementsByTagName("km");
        String km = nodes.item(0).getTextContent();
        double dkm = Double.parseDouble(km);
        String river = access.getRiver();
        atWriter.write(writer, context.getMeta(), river, dkm);
        writer.close();
    }

    @Override
    protected void writeCSVData(CSVWriter writer) throws IOException {
        // The concrete writer is used to write csv data.
    }

    @Override
    protected void writePDF(OutputStream out) {
        // Implement me!
    }

    @Override
    protected void addData(Object data) {
        // Nothing to do here.
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
